<template>
  <div v-show="chat.previewImage"
       class="fixed top-0 left-0 bg-gray-900 h-full w-full bg-opacity-95"
       style="z-index: 9999">
    <div class="relative flex flex-col">
      <header class="h-14 relative flex p-4 justify-between items-center w-full">
        <div></div>
        <div class="relative flex items-center gap-2">
          <button
              @click="downloadImg(helpers.generateImage(firstImage.object?.image?.image_url))"
              class="relative block btn btn-circle btn-ghost p-2">
            <svg xmlns="http://www.w3.org/2000/svg" class="w-full h-full fill-current text-white" viewBox="0 0 24 24">
              <path d="M5,20H19V18H5M19,9H15V3H9V9H5L12,16L19,9Z"/>
            </svg>
          </button>
          <button @click="closePreviewImage" class="relative block btn btn-circle btn-ghost p-2">
            <svg class="w-full h-full fill-current text-white" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 24 24">
              <path
                  d="M13.46,12L19,17.54V19H17.54L12,13.46L6.46,19H5V17.54L10.54,12L5,6.46V5H6.46L12,10.54L17.54,5H19V6.46L13.46,12Z"/>
            </svg>
          </button>
        </div>
      </header>
      <div style="height: calc(100vh - 9.5rem)" class="relative block w-full overflow-hidden">
        <div class="relative block h-full w-full text-center p-12">
          <div
              @click="nextPrevImage('prev')"
              class="absolute left-3 top-1/2 transform -translate-y-1/2 my-auto h-10 w-10 bg-white cursor-pointer rounded-full p-2">
            <svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 24 24">
              <path d="M20,11V13H8L13.5,18.5L12.08,19.92L4.16,12L12.08,4.08L13.5,5.5L8,11H20Z"/>
            </svg>
          </div>
          <div ref="zoomImage" class="relative flex h-full zoom w-full items-center overflow-hidden justify-center">
            <img :src="firstImage?.object?.image?.image_url"
                 class="w-auto select-none  h-full place-items-center overflow-hidden block object-contain"/>
          </div>
          <div
              @click="nextPrevImage('next')"
              class="absolute right-3 top-1/2 transform -translate-y-1/2 my-auto h-10 w-10 bg-white cursor-pointer rounded-full p-2">
            <svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 24 24">
              <path d="M4,11V13H16L10.5,18.5L11.92,19.92L19.84,12L11.92,4.08L10.5,5.5L16,11H4Z"/>
            </svg>
          </div>
        </div>
      </div>
      <footer
          class="h-24 relative block z-40 py-2 bg-gray-900 bg-opacity-50 border-t border-gray-800 overflow-x-hidden overflow-y-hidden w-full">
        <div class="relative flex flex-shrink-0 gap-2 h-full w-full overflow-y-hidden">
          <perfect-scrollbar
              ref="scroll" class="overflow-y-hidden flex p-0 flex-row flex-1 gap-1 relative">
          <div v-for="(image,index) in imageData"
               @click="changeImage(image)"
               class="h-full aspect-square cursor-pointer flex flex-shrink-0 justify-center transition duration-200 group overflow-hidden relative bg-gray-200 border border-gray-100">
            <img :src="image?.object?.image?.image_url" draggable="false"
                 class="absolute select-none top-1/2 left-1/2 min-w-full min-h-full transform -translate-y-1/2 -translate-x-1/2 m-auto object-cover"/>
            <span
                class="absolute left-0 transition duration-200 top-0 w-full h-full group-hover:bg-black group-hover:bg-opacity-20"></span>
          </div>
          </perfect-scrollbar>
        </div>
      </footer>
    </div>
  </div>
</template>
<script setup lang="ts">

import {onMounted, ref, watch} from "vue";
import {useChatListStore} from "../../stores/chat_list";
import {useArrayFindIndex} from "@vueuse/core";
import helpers from "../../utils/helpers";

const chat = useChatListStore()
const firstImage = ref(null)
const zoomImage = ref()
const imageData  = ref([])
const zoomLevel = ref(1)
const downloadLoading = ref(false)
const changeImage = (image_url) => {
  firstImage.value = image_url
  const zoomElement = document.querySelector(".zoom > img");
  zoomElement["style"].transform = `scale(1)`;
}

const nextPrevImage = (type) => {
  if (firstImage.value) {
    const currentImage = firstImage.value
    const currentImageId = currentImage.id
    const findIndex = chat.chatImages.findIndex(i => i.id === currentImageId)
    let currentIndex
    if (type === 'next') {
      let increaseIndex = findIndex + 1
      currentIndex = increaseIndex >= chat.chatImages.length ? 0 : increaseIndex
    } else {
      let decreaseIndex = findIndex - 1
      currentIndex = findIndex <= 0 ? chat.chatImages.length - 1 : decreaseIndex
    }
    firstImage.value = chat.chatImages[currentIndex]
  }
}

onMounted(() => {
  const zoomElement = document.querySelector(".zoom > img");
  let zoom = 1;
  const ZOOM_SPEED = 0.1;
  document.addEventListener("wheel", function (e) {
    if (e.deltaY > 0) {
      if (zoomLevel.value >= 1) {
        zoomElement["style"].transform = `scale(${zoomLevel.value -= ZOOM_SPEED})`;
      }
    } else {
      zoomElement["style"].transform = `scale(${zoomLevel.value += ZOOM_SPEED})`;
    }
  });

  if (chat.chatImages.length > 0) {
    firstImage.value = chat.chatImages[0]
  }


})

watch(() => chat.chatImages, (val, oldValue) => {
  firstImage.value = val[0] ?? null
})

watch(() => chat.previewImage, (val, oldValue) => {
  if (val == false) {
    const zoomElement = document.querySelector(".zoom");
    zoomElement["style"].transform = `scale(1)`;
    zoomLevel.value = 1
  }
  imageData.value = chat.chatImages

  document.onkeydown = function (e) {
    switch (e.key) {
      case 'ArrowUp':
        // up arrow
        break;
      case 'ArrowDown':
        // down arrow
        break;
      case 'ArrowLeft':
        nextPrevImage("prev")
        // left arrow
        break;
      case 'ArrowRight':
        nextPrevImage("next")
    }
  };
})

watch(() => chat.previewImageData, (val, oldValue) => {
  // console.log(val, "ISI PREVIEW IMAGE DAT")
  firstImage.value = val
})
const downloadImg = async (url) => {
  downloadLoading.value = true
  const link = document.createElement('a');
  link.href = url;
  link.setAttribute('download', url.substring(url.lastIndexOf('/') + 1)); //or any other extension
  document.body.appendChild(link);
  link.click();
  downloadLoading.value = false
}

const closePreviewImage = () => {
  chat.previewImage = false
  chat.previewImageData = null
}
</script>